# -*- coding: utf-8 -*-
import emoji.unicode_codes as codes
import re
from itertools import repeat
from pynlple.data.datasource import TsvDataframeSource
from pynlple.module import abs_path, append_paths

DEFAULT_EMOJI_CLASS_NAME = 'emoji'
DEFAULT_COMPONENT_CLASS_NAME = 'emoji-component'
CLASS_NAME_DELIMITER = '_'


class UnicodeEmojiProvider(object):

    def __iter__(self):
        ws_patt = re.compile(r'\s+')
        source = codes.EMOJI_UNICODE.items()
        emojis = sorted(source, key=len, reverse=True)
        for name, emoji_ in emojis:
            yield (ws_patt.sub('', emoji_), 'emoji:' + name)


class UnicodeEmojiProviderV11(object):


    def __init__(self, full_names=True, class_name=DEFAULT_EMOJI_CLASS_NAME):
        self.full_names = full_names
        self.class_name = class_name

        path = append_paths(abs_path(__file__, 'data'), 'emoji', 'emojis.txt')
        emoji_df = TsvDataframeSource(path, encoding='utf-8').get_dataframe()
        emoji_df.loc[:,'len'] = emoji_df.loc[:,'char'].apply(len)
        emoji_df.sort_values(by=['len', 'char'], ascending=False, inplace=True)

        self.emojis = list(emoji_df.loc[:,'char'].tolist())
        if self.full_names and self.class_name:
            self.names = list(map(lambda e: self.class_name + CLASS_NAME_DELIMITER + e, emoji_df.loc[:,'cldr_alias']))
        elif self.full_names and not self.class_name:
            self.names = list(emoji_df.loc[:,'cldr_alias'].tolist())
        else:
            self.names = repeat(self.class_name if self.class_name else DEFAULT_EMOJI_CLASS_NAME)

    def __iter__(self):
        return iter(zip(self.emojis, self.names))


class UnicodeClusteredEmojiProviderV11(object):

    def __init__(self, class_name=DEFAULT_EMOJI_CLASS_NAME):
        self.class_name = class_name

        path = append_paths(abs_path(__file__, 'data'), 'emoji', 'emojis_clustered.txt')
        emoji_df = TsvDataframeSource(path, encoding='utf-8').get_dataframe()
        emoji_df.loc[:,'len'] = emoji_df.loc[:,'char'].apply(len)
        emoji_df.sort_values(by=['len', 'char'], ascending=False, inplace=True)

        self.emojis = list(emoji_df.loc[:,'char'].tolist())
        if self.class_name:
            prefix = self.class_name
        else:
            prefix = DEFAULT_EMOJI_CLASS_NAME
        self.cluster_names = list(map(lambda c: prefix + CLASS_NAME_DELIMITER + str(c), emoji_df.loc[:,'cluster']))

    def __iter__(self):
        return iter(zip(self.emojis, self.cluster_names))


class UnicodeEmojiComponentProviderV11(object):

    def __init__(self):
        path = append_paths(abs_path(__file__, 'data'), 'emoji', 'emoji_components.txt')
        emoji_df = TsvDataframeSource(path, encoding='utf-8').get_dataframe()
        emoji_df.loc[:,'len'] = emoji_df.loc[:,'char'].apply(len)
        emoji_df.sort_values(by=['len', 'char'], ascending=False, inplace=True)
        self.emojis = list(emoji_df.loc[:,'char'].tolist())
        self.names = list(emoji_df.loc[:,'class'].tolist())

    def __iter__(self):
        return iter(zip(self.emojis, self.names))



# -*- coding: utf-8 -*-
import unittest

from pynlple.exceptions import DataSourceException
from pynlple.processing.dictionary import FileFolderTokenMapping
from ..module import abs_path as path

from ..processing.dictionary import DictionaryLookUp
from ..processing.text import DictionaryBasedTagger

class TaggerTest(unittest.TestCase):

    def setUp(self):
        self.mappings = [
            ('a b c', 'class1'),
            ('a b', 'class11'),
            ('a c', 'class1'),
            ('b c', 'class11'),
            ('a a a', 'class2'),
            ('a a', 'class21'),
            ('b b b', 'class2'),
            ('c c c', 'class2'),
            ('a b c d', 'class3'),
            ('a b c e', 'class3'),
        ]
        self.dictionary = DictionaryLookUp(self.mappings)

    def test_should_result_be_empty(self):
        tagger = DictionaryBasedTagger(DictionaryLookUp([]))
        tokens = 'a b c a b a a a c c c a b c d a b c e'.split()
        self.assertEqual(0, len(tagger.get_tagged_entities(tokens)))

    def test_should_match_one_result(self):
        tagger = DictionaryBasedTagger(self.dictionary)
        tokens = 'a c a b'.split()
        self.assertEqual([(0, 2, 'class1'), (2, 4, 'class11')], tagger.get_tagged_entities(tokens))

    def test_should_match_some_results(self):
        tagger = DictionaryBasedTagger(self.dictionary)
        tokens = 'c c c a'.split()
        self.assertEqual((0, 3, 'class2'), tagger.get_tagged_entities(tokens)[0])

    def test_should_match_many_longer_parts_results(self):
        tagger = DictionaryBasedTagger(self.dictionary)
        tokens = 'a b c b b b a b c e a a a'.split()
        self.assertEqual([(0, 3, 'class1'),
                          (3, 6, 'class2'),
                          (6, 10, 'class3'),
                          (10, 13, 'class2')],
                         tagger.get_tagged_entities(tokens, False))

    def test_should_match_all_and_short_parts_results(self):
        tagger = DictionaryBasedTagger(self.dictionary)
        tokens = 'a b c b b b a b c e a a a'.split()
        self.assertEqual({(0, 3, 'class1'),
                          (0, 2, 'class11'),
                          (1, 3, 'class11'),
                          (3, 6, 'class2'),
                          (6, 10, 'class3'),
                          (6, 8, 'class11'),
                          (7, 9, 'class11'),
                          (6, 9, 'class1'),
                          (10, 13, 'class2'),
                          (10, 12, 'class21'),
                          (11, 13, 'class21')},
                         set(tagger.get_tagged_entities(tokens, True)))


class FileFolderTokenMappingTest(unittest.TestCase):

    def test_should_get_mappings_from_whole_data_folder_extension_txt(self):
        mapper = FileFolderTokenMapping(data_folder=path(__file__, 'data/dictionary'), extension='.txt')

        mappings = {
            (path(__file__, 'data/dictionary/file1.txt'), 'file1'),
            (path(__file__, 'data/dictionary/sub1data/sub1file1.txt'), 'sub1data/sub1file1'),
            (path(__file__, 'data/dictionary/sub1data/sub1file2.txt'), 'sub1data/sub1file2'),
            (path(__file__, 'data/dictionary/sub1data/sub1sub1data/sub1sub1file1.txt'), 'sub1data/sub1sub1data/sub1sub1file1'),
            (path(__file__, 'data/dictionary/sub2data/sub2file1.txt'), 'sub2data/sub2file1'),
            (path(__file__, 'data/dictionary/sub2data/sub2file2.txt'), 'sub2data/sub2file2'),
        }
        self.assertEqual(mappings, mapper.get_file_tag_mappings())

    def test_should_get_few_mappings_from_whole_data_folder_NOT_recursive(self):
        mapper = FileFolderTokenMapping(data_folder=path(__file__, 'data/dictionary'), recursive=False)

        mappings = {
            (path(__file__, 'data/dictionary/file1.txt'), 'file1'),
        }
        self.assertEqual(mappings, mapper.get_file_tag_mappings())

    def test_should_get_few_mappings_from_whole_sub_data_folder_NOT_recursive_txt(self):
        mapper = FileFolderTokenMapping('sub1data', data_folder=path(__file__, 'data/dictionary'), recursive=False, extension='.txt')

        mappings = {
            (path(__file__, 'data/dictionary/sub1data/sub1file1.txt'), 'sub1data/sub1file1'),
            (path(__file__, 'data/dictionary/sub1data/sub1file2.txt'), 'sub1data/sub1file2'),
        }
        self.assertEqual(mappings, mapper.get_file_tag_mappings())

    def test_should_get_mapping_for_file_no_extension_extension_not_specified(self):
        mapper = FileFolderTokenMapping('no_extension', data_folder=path(__file__, 'data/dictionary'), extension=None)

        mappings = {
            (path(__file__, 'data/dictionary/no_extension'), 'no_extension')
        }
        self.assertEqual(mappings, mapper.get_file_tag_mappings())

    def test_should_get_mappings_from_whole_data_folder_extension_any(self):
        mapper = FileFolderTokenMapping(data_folder=path(__file__, 'data/dictionary'), extension=None)

        mappings = {
            (path(__file__, 'data/dictionary/no_extension'), 'no_extension'),
            (path(__file__, 'data/dictionary/file1.txt'), 'file1'),
            (path(__file__, 'data/dictionary/sub1data/sub1file1.txt'), 'sub1data/sub1file1'),
            (path(__file__, 'data/dictionary/sub1data/sub1file2.txt'), 'sub1data/sub1file2'),
            (path(__file__, 'data/dictionary/sub1data/sub1file1.csv'), 'sub1data/sub1file1'),
            (path(__file__, 'data/dictionary/sub1data/sub1sub1data/sub1sub1file1.txt'), 'sub1data/sub1sub1data/sub1sub1file1'),
            (path(__file__, 'data/dictionary/sub2data/sub2file1.txt'), 'sub2data/sub2file1'),
            (path(__file__, 'data/dictionary/sub2data/sub2file1.csv'), 'sub2data/sub2file1'),
            (path(__file__, 'data/dictionary/sub2data/sub2file2.txt'), 'sub2data/sub2file2'),
        }
        self.assertEqual(mappings, mapper.get_file_tag_mappings())

    def test_should_get_mappings_via_dict_keys_and_values(self):
        mapper = FileFolderTokenMapping({'sub1data': ['sub1file1.txt', 'sub1file2.txt'], 'sub2data': ['sub2file1.txt']},
                                        data_folder=path(__file__, 'data/dictionary'))

        mappings = {
            (path(__file__, 'data/dictionary/sub1data/sub1file1.txt'), 'sub1data/sub1file1'),
            (path(__file__, 'data/dictionary/sub1data/sub1file2.txt'), 'sub1data/sub1file2'),
            (path(__file__, 'data/dictionary/sub2data/sub2file1.txt'), 'sub2data/sub2file1'),
        }
        self.assertEqual(mappings, mapper.get_file_tag_mappings())

    def test_should_get_mappings_via_filenames_without_extension_extension_specified(self):
        mapper = FileFolderTokenMapping({'sub1data': ['sub1file1', 'sub1file2'], 'sub2data': ['sub2file1']},
                                        data_folder=path(__file__, 'data/dictionary'), extension='.txt')

        mappings = {
            (path(__file__, 'data/dictionary/sub1data/sub1file1.txt'), 'sub1data/sub1file1'),
            (path(__file__, 'data/dictionary/sub1data/sub1file2.txt'), 'sub1data/sub1file2'),
            (path(__file__, 'data/dictionary/sub2data/sub2file1.txt'), 'sub2data/sub2file1'),
        }
        self.assertEqual(mappings, mapper.get_file_tag_mappings())

    def test_should_raise_exception_on_filenames_without_extension_extension_not_specified(self):
        self.assertRaises(DataSourceException, lambda: FileFolderTokenMapping({'sub1data': ['sub1file1', 'sub1file2'], 'sub2data': ['sub2file1']},
                                        data_folder=path(__file__, 'data/dictionary'), extension=None))

    def test_should_get_mappings_via_dict_keys_nested(self):
        mapper = FileFolderTokenMapping({'sub1data': {'sub1sub1data': 'sub1sub1file1.txt'}}, data_folder=path(__file__, 'data/dictionary'))

        mappings = {
            (path(__file__, 'data/dictionary/sub1data/sub1sub1data/sub1sub1file1.txt'), 'sub1data/sub1sub1data/sub1sub1file1')
        }
        self.assertEqual(mappings, mapper.get_file_tag_mappings())

    def test_should_raise_exception_on_wrong_path(self):
        self.assertRaises(DataSourceException, lambda: FileFolderTokenMapping('this/path/does/not/exists.txt',
                                                                              data_folder=path(__file__, 'data')))

    def test_should_init_lookup_from_mapper(self):
        mapper = FileFolderTokenMapping('sub1data/sub1sub1data/sub1sub1file1.txt', data_folder=path(__file__, 'data/dictionary'))
        dictionary_lookup = DictionaryLookUp(mapper)

        self.assertIsNotNone(dictionary_lookup)
        self.assertEquals('sub1data/sub1sub1data/sub1sub1file1', dictionary_lookup['a'])


class FileFolderTokenMappingReadTest(unittest.TestCase):

    def setUp(self):
        pass

    def test_should_read_first_and_third_lines(self):
        name = 'escape'
        import io
        with io.open('data/' + name + '.txt', 'rt', encoding='utf-8') as file:
            read_file = [(line.split(), name) for line in file]
            expected_mappings = [read_file[0], read_file[2]]

            mapper = FileFolderTokenMapping(name, data_folder=path(__file__, 'data'))
            self.assertEqual(expected_mappings, list(mapper))


class DictionaryLookUpPreprocessingTest(unittest.TestCase):

    def setUp(self):
        self.mappings = [
            ('a B c', 'class121'),
            ('a b C', 'class112'),
            ('A b C', 'class212')
        ]

    def test_should_apply_while_initing(self):
        preprocessing = lambda x: x.lower()
        dict = DictionaryLookUp(self.mappings, preprocessing)

        self.assertIsNotNone(dict['a b c'])
        self.assertEqual('class212', dict['a b c'])

    def test_should_apply_while_referring(self):
        preprocessing = lambda x: x.lower()
        dict = DictionaryLookUp(self.mappings, preprocessing)

        self.assertIsNotNone(dict['a B C'])
        self.assertEqual('class212', dict['A b c'])
        self.assertEqual(dict['A b C'], dict['a B c'])


class DictionaryLookUpTest(unittest.TestCase):

    def setUp(self):
        self.mappings = [
            ('a b c', 'class1'),
            ('a b', 'class11'),
            ('a c', 'class1'),
            ('b c', 'class1'),
            ('a a a', 'class2'),
            ('a a', 'class21'),
            ('b b b', 'class2'),
            ('c c c', 'class2'),
            ('a b c d', 'class3'),
            ('a b c e', 'class3'),
        ]

    def test_should_init_from_strings(self):
        dictionary = DictionaryLookUp(self.mappings)
        self.assertIsNotNone(dictionary)

    def test_should_init_from_tokens(self):
        dictionary = DictionaryLookUp([(tokens.split(), classes) for (tokens, classes) in self.mappings])
        self.assertIsNotNone(dictionary)

    def test_should_contain_simple_mapping(self):
        dictionary = DictionaryLookUp(self.mappings)
        self.assertIsNotNone(dictionary.get_tag(self.mappings[7][0]))

    def test_should_contain_sub_mapping(self):
        dictionary = DictionaryLookUp(self.mappings)
        self.assertIsNotNone(dictionary.get_tag(self.mappings[1][0]))

    def test_should_contain_mega_mapping(self):
        dictionary = DictionaryLookUp(self.mappings)
        self.assertIsNotNone(dictionary.get_tag(self.mappings[8][0]))

    def test_should_not_contain_mapping(self):
        dictionary = DictionaryLookUp(self.mappings)
        self.assertIsNone(dictionary.get_tag(' '.join([tokens for tokens, _ in self.mappings])))

    def test_should_contain_correct_tag_for_simple_mapping(self):
        dictionary = DictionaryLookUp(self.mappings)
        self.assertEqual(dictionary.get_tag(self.mappings[7][0]), self.mappings[7][1])

    def test_should_contain_correct_tag_for_sub_mapping(self):
        dictionary = DictionaryLookUp(self.mappings)
        self.assertEqual(dictionary.get_tag(self.mappings[1][0]), self.mappings[1][1])

    def test_should_contain_correct_tag_for_mega_mapping(self):
        dictionary = DictionaryLookUp(self.mappings)
        self.assertEqual(dictionary.get_tag(self.mappings[8][0]), self.mappings[8][1])

    def test_should_rewrite_class(self):
        new_mappings = list(self.mappings)
        new_mappings.append((self.mappings[len(self.mappings) - 1][0],self.mappings[len(self.mappings) - 1][1][::-1]))
        dictionary = DictionaryLookUp(new_mappings)
        self.assertEqual(dictionary.get_tag(self.mappings[len(self.mappings) - 1][0]), new_mappings[len(new_mappings) - 1][1])
        self.assertNotEqual(dictionary.get_tag(self.mappings[len(self.mappings) - 1][0]), self.mappings[len(self.mappings) - 1][1])

    def test_should_contain_correct_tag_for_simple_mapping_via_getitem(self):
        dictionary = DictionaryLookUp(self.mappings)
        self.assertEqual(dictionary[self.mappings[7][0]], self.mappings[7][1])

    def test_should_contain_simple_mapping_via_getitem(self):
        dictionary = DictionaryLookUp(self.mappings)
        self.assertIsNotNone(dictionary[self.mappings[7][0]])

    def test_should_add_simple_mapping_via_setitem(self):
        dictionary = DictionaryLookUp({})
        dictionary['new mapping'] = 'new_class'
        self.assertIsNotNone(dictionary['new mapping'])
        self.assertEqual(dictionary['new mapping'], 'new_class')

    def test_should_get_longest(self):
        dictionary = DictionaryLookUp(self.mappings)
        self.assertEqual(max([len(token.split()) for token, _ in self.mappings]), dictionary.get_longest_sequence())

    def test_should_get_longest_zero(self):
        dictionary = DictionaryLookUp([])
        self.assertEqual(0, dictionary.get_longest_sequence())

    def test_should_get_longest_multiple_alternatives(self):
        mappings = [('a a a a a a a a a a b', 'c'), ('a a a a a a a a a a c', 'c2')]
        dictionary = DictionaryLookUp(mappings)
        self.assertEqual(max([len(token.split()) for token, _ in mappings]), dictionary.get_longest_sequence())

    def test_should_get_known_classes_empty(self):
        dictionary = DictionaryLookUp([])
        self.assertEqual(0, len(dictionary.get_known_classes()))

    def test_should_get_known_classes(self):
        dictionary = DictionaryLookUp(self.mappings)
        self.assertEqual(sorted(list({c for _, c in self.mappings})), sorted(list(dictionary.get_known_classes())))


# -*- coding: utf-8 -*-
import types
from operator import attrgetter as _attrgetter
from operator import itemgetter as _itemgetter
from functools import partial
from math import ceil
from numpy import arange, repeat, insert, array, cumsum


def __get_utf16_code_length(symbol):
    return int((len(symbol.encode(encoding='utf-16')) - 2) / 2.0)


def calculate_text_position_utf16_bias(python_text):
    utf16_symbol_lengths = list(map(__get_utf16_code_length, python_text))
    p_text_len = len(utf16_symbol_lengths)
    arr = arange(0, p_text_len)
    for i in range(p_text_len - 1, -1, -1):
        add_len = utf16_symbol_lengths[i] - 1
        if add_len > 0:
            arr = insert(arr, i, repeat(i, add_len))
    return arr.tolist()


def calculate_text_position_towards_utf16_bias(python_text):
    utf16_symbol_lengths = array(list(map(__get_utf16_code_length, python_text)))
    utf16_symbol_lengths = insert(utf16_symbol_lengths, 0, 0)
    return cumsum(utf16_symbol_lengths)


def __wrapped_attrgetter(obj, attrname, default):
    try:
        return _attrgetter(attrname)(obj)
    except AttributeError:
        return default


def __wrapped_itemgetter(obj, items, default):
    try:
        return _itemgetter(items)(obj)
    except IndexError:
        return default
    except KeyError:
        return default


def attrgetter(attrname, default=None):
    return partial(__wrapped_attrgetter, attrname=attrname, default=default)


def itemgetter(items, default=None):
    return partial(__wrapped_itemgetter, items=items, default=default)


def csr_apply_1d(csr_matrix, other_1d_matrix, np_func, copy=False, batch=512,  **kwargs):
    xs, ys = csr_matrix.nonzero()
    other_1d_matrix_xs = other_1d_matrix[xs].reshape((-1))
    if copy:
        target_matrix = csr_matrix.copy()
    else:
        target_matrix = csr_matrix
    slices = ceil(target_matrix.data.shape[0] / batch)
    for i in range(slices):
        np_func(csr_matrix.data[i * batch:(i + 1) * batch],
                other_1d_matrix_xs[i * batch:(i + 1) * batch],
                out=target_matrix.data[i * batch:(i + 1) * batch],
                **kwargs)
    return target_matrix


class HashHelper(object):

    @staticmethod
    def nice_hash(o):
        """
        Makes a hash from a dictionary, list, tuple or set to any level, that contains
        only other hashable types (including any lists, tuples, sets, and
        dictionaries).
        Taken from: http://stackoverflow.com/a/8714242
        """
        import copy

        if isinstance(o, (set, tuple, list)):
            return tuple([HashHelper.nice_hash(e) for e in o])
        elif isinstance(o, dict):
            new_o = copy.deepcopy(o)
            for k, v in new_o.items():
                new_o[k] = HashHelper.nice_hash(v)
            return hash(tuple(frozenset(sorted(new_o.items()))))
        else:
            return hash(o)


class DictionaryHelper(object):

    @staticmethod
    def get_value(dictionary, path_dictionary):
        """
        Retrieve the value from nested dictionary structure by nested key.
        Supports lists, sets, tuples, dictionaries and all nested into themselves.

        :param dict dictionary: nested dictionary with complex structure
        :param object path_dictionary: nested (or not) path in nested dictionary
         to retrieve data by
        :return: the sub-dictionary found by the path in nested dictionary
        :rtype: object
        :raises ValueError: raises :class:ValueError if nothing can be found by specified path
        :raises KeyError: raises :class:KeyError if specified path is incorrect,
        containing illogical multiple choices or conditions.
        """
        return DictionaryHelper.__inner_get(dictionary, path_dictionary)

    @staticmethod
    def __inner_get(dictionary, path_dictionary):
        """"""
        # Check if we have set or single value in path
        if type(path_dictionary) is list or type(path_dictionary) is tuple:
            raise KeyError('Lists and tuples are not supported in path_dictionary')
        elif type(path_dictionary) is set:
            if len(path_dictionary) > 1:
                raise KeyError('Condition path has multiple conditions')
            for cond in path_dictionary:
                return dictionary[cond]
        elif type(path_dictionary) is str:
            return dictionary[path_dictionary]
        elif type(path_dictionary) is dict:
            conditions = path_dictionary.keys()
            if len(conditions) > 1:
                raise KeyError('Condition path has multiple conditions')
            elif len(conditions) == 0:
                raise KeyError('Condition path has zero conditions')
            if conditions[0] not in dictionary:
                    raise ValueError('Dictionary did not contain condition sub path')
            else:
                return DictionaryHelper.__inner_get(dictionary[conditions[0]], path_dictionary[conditions[0]])
        else:
            raise KeyError('Type ' + type(path_dictionary) + ' is not supported by dictionary path')

    @staticmethod
    def matches(condition, state):
        return DictionaryHelper.__inner_match(condition, state)

    @staticmethod
    def __inner_match(condition, state):
        """Method matches"""
        if type(condition) is dict:
            for ck in condition.keys():
                if ck not in state:
                    return False
                else:
                    if not DictionaryHelper.__inner_match(condition[ck], state[ck]):
                        return False
            return True
        elif type(condition) is list or type(condition) is set or type(condition) is tuple:
            for ck in condition:
                if not DictionaryHelper.__inner_match(ck, state):
                    return False
            return True
        elif isinstance(condition, types.FunctionType):
            return condition(state)
        else:
            if type(state) is dict:
                return condition in state
            elif type(state) is list or type(condition) is set or type(condition) is tuple:
                return condition in state
            else:
                return condition == state



import json

class Feature:
    """Class for feature representation
        name - feature's name
        param - param of classifier corresponding to feature
        value - feature's value
        weight - param * value
    """

    def __init__(self, name, param, value=1):
        self.name = name
        self.param = param
        self.value = value
        self.weight = param * value

    def __str__(self):
        return "\t%-30s\t\t%.4f\t%.4f\t%.4f" % (self.name, self.weight, self.param ,self.value)

    def __lt__(self, other):
        return self.weight < other.weight

    def __repr__(self):
        return json.dumps(self, default=lambda o: o.__dict__, indent=4)


class FeatureExploration:
    """Class for expolre features in model
        vectorizer, classifer, preprocessor - steps of model pipeline
            preprocessor: process text
            vectorizer: vectorize proceseed text
            classifer: classify vector
    """

    def __init__(self, vectorizer, classifer, preprocessor=None):
        self.vectorizer = vectorizer
        self.classifer = classifer
        self.preprocessor = preprocessor
        self.features = list(zip(vectorizer.get_feature_names(), classifer.coef_[0]))

    def find_features(self, text):
        """find features in text

        Args:
             text: raw text for analysis
        Returns:
            initial_text: preprocessed text
            features: finded features

        """
        finded_features = []
        if self.preprocessor:
            text = self.preprocessor.transform([text])[0]
        for feature_number, feature_coef in self.vectorizer.transform([text]).todok().items():
            finded_feature = self.features[feature_number[1]]
            finded_features.append(Feature(finded_feature[0], finded_feature[1], feature_coef))
        finded_features = sorted(finded_features, reverse=True)
        return {
            'initial_text': text,
            'features': finded_features
        }

    def show_features(self, text):
        """print finded features from text

        Args:
             text: raw text for analysis
        """
        rez = self.find_features(text)
        print(rez['initial_text'])
        for f in rez['features']:
            print(f)

    def show_most_informative_features(self, n=20):
        """print n most good and bad features of model

        Args:
             n: how many top features should be showed
        """
        feature_names = self.vectorizer.get_feature_names()
        coefs_with_fns = sorted(zip(self.classifer.coef_[0], feature_names))
        top = zip(coefs_with_fns[:n], coefs_with_fns[:-(n + 1):-1])
        for (coef_1, fn_1), (coef_2, fn_2) in top:
            print("\t%.4f\t%-15s\t\t%.4f\t%-15s" % (coef_1, fn_1, coef_2, fn_2))



# -*- coding: utf-8 -*-
import unittest
from pynlple.data.corpus import StackingSource, DeduplicatingFixedCacheSizeSource


class StackingSourceTest(unittest.TestCase):

    def setUp(self):
        self.source1 = [
            '1', '2', '3'
        ]
        self.source2 = [
            4, 5, 6
        ]

    def test_should_return_all_items_consequently(self):
        stacking_source = StackingSource([self.source1, self.source2])
        expected_items = ['1', '2', '3', 4, 5, 6]

        self.assertEqual(expected_items, [item for item in stacking_source])


class DeduplicatingFixedCacheSizeSourceTest(unittest.TestCase):

    def setUp(self):
        self.source = [1, 1, 2, 2, 3, 1, 2, 4, 5, 1, 1, 2, 2, 3, 3, 4, 6, 7]

    def test_should_yield_deduplicated_no_cache_overflow(self):
        dedup_source = DeduplicatingFixedCacheSizeSource(self.source, cache_size=15)

        expected_result = [1, 2, 3, 4, 5, 6, 7]
        self.assertEqual(expected_result, [i for i in dedup_source])

    def test_should_yield_partially_deduplicated_cache_overflow(self):
        dedup_source = DeduplicatingFixedCacheSizeSource(self.source, cache_size=4, refresh=False)

        expected_result = [1, 2, 3, 4, 5, 1, 2, 3, 4, 6, 7]
        self.assertEqual(expected_result, [i for i in dedup_source])

    def test_should_yield_partially_deduplicated_cache_overflow_refreshing(self):
        dedup_source = DeduplicatingFixedCacheSizeSource(self.source, cache_size=4, refresh=True)

        expected_result = [1, 2, 3, 4, 5, 3, 4, 6, 7]
        self.assertEqual(expected_result, [i for i in dedup_source])

    def test_should_yield_deduplicated_feature_extrator(self):
        dedup_source = DeduplicatingFixedCacheSizeSource(self.source, cache_size=15, feature_extractor=lambda x: x if x <= 4 else 4)

        expected_result = [1, 2, 3, 4]
        self.assertEqual(expected_result, [i for i in dedup_source])


# -*- coding: utf-8 -*-
import re
from .token import TOKEN_TYPES


class ITokenizer(object):

    def tokenize(self, sentence):
        return None


class RegexTokenizer(ITokenizer):

    def __init__(self, regex_query, ignore_case=False):
        self.__p = re.compile(regex_query, re.IGNORECASE if ignore_case else 0)

    def tokenize(self, sentence):
        return self.__p.split(sentence)


class WSTokenizer(RegexTokenizer):

    def __init__(self, regex_query=r'[\f   \t\v]+'):
        super().__init__(regex_query=regex_query)


class RuleTokenizer(ITokenizer):

    def __init__(self):
        self.__p = re.compile(r'' + r'|'.join(TOKEN_TYPES) + r'', re.IGNORECASE)

    def tokenize(self, sentence):
        return list(matches.string[matches.span()[0]:matches.span()[1]] for matches in self.__p.finditer(sentence))


class TokenTagger(object):

    def get_tagged_entities(self, tokens, all_=True):
        return None


class DictionaryBasedTagger(TokenTagger):

    def __init__(self, dictionary_look_up):
        self.lookup = dictionary_look_up

    def get_classes(self):
        return sorted(list(self.lookup.get_known_classes()))

    def get_tagged_entities(self, tokens, all_=False):
        max_ = self.lookup.get_longest_sequence()
        entities = list()
        toks = list(tokens)
        len_ = len(toks)
        i = 0
        while i < len_:
            j = i + max_ if i + max_ <= len_ else len_
            while j > i:
                class_ = self.lookup[toks[i:j]]
                if class_ is not None:
                    entities.append((i, j, class_))
                    if not all_:
                        i = j - 1
                        break
                j -= 1
            i += 1
        return entities


class TokenFilter(object):

    def filter(self, tokens):
        """
        Filter out tokens according to specific rules.

        :param list tokens: string list of tokens to filter
        :return: filtered list of tokens
        :rtype: list
        """
        return None


class StackingFilter(TokenFilter):
    """Class for continuous stacking and applying filters in a natural order."""

    def __init__(self, filter_list=list()):
        self.filters = list(filter_list)

    def append_filter(self, filter):
        self.filters.append(filter)

    def prepend_filter(self, filter):
        self.filters.insert(0, filter)

    def filter(self, tokens):
        for filter in self.filters:
            tokens = filter.filter(tokens)
        return tokens


class ClassTokenFilter(TokenFilter):

    def __init__(self, tagger, filtered_classes=None):
        self.__tagger = tagger
        self.filtered_classes = filtered_classes

    def filter(self, tokens):
        toks = list(tokens)
        tags = self.__tagger.get_tagged_entities(toks, all_=False)
        if self.filtered_classes:
            tags = filter(lambda t: t[2] in self.filtered_classes, tags)
        invalid_ids = set(i for start, end, _ in tags for i in range(start, end))
        return [toks[i] for i in range(0, len(toks)) if i not in invalid_ids]





# -*- coding: utf-8 -*-
import unittest
from pynlple.processing.mention import cut_text_with_highlights_if_present


class MentionUtilityTest(unittest.TestCase):

    def test_should_not_cut_text_w_highlights(self):
        text = 'this text contains highlights\r\nand new lines so that i can test the utility method\r\n' \
               'some additional lines are added for longer text'
        highlights = [{'start': 5, 'end': 9}, {'start': 45, 'end': 47}, {'start': 100, 'end': 105}]

        expected_text = text
        expected_highlights = highlights
        result_text, result_highlights = cut_text_with_highlights_if_present(text, highlights)
        self.assertEqual(expected_text, result_text)
        self.assertEqual(expected_highlights, result_highlights)

    def test_should_not_cut_text_w_no_highlights(self):
        text = 'this text contains highlights\r\nand new lines so that i can test the utility method\r\n' \
               'some additional lines are added for longer text'
        highlights = []

        expected_text = text
        expected_highlights = highlights
        result_text, result_highlights = cut_text_with_highlights_if_present(text, highlights)
        self.assertEqual(expected_text, result_text)
        self.assertEqual(expected_highlights, result_highlights)

    def test_should_cut_text_w_highlights_in_title(self):
        text = 'this text contains highlights\r\nand new lines so that i can test the utility method\r\n' \
               'some additional lines are added for longer text'
        highlights = [{'start': 5, 'end': 9}]

        expected_text = text[:29]
        expected_highlights = highlights
        result_text, result_highlights = cut_text_with_highlights_if_present(text, highlights)
        self.assertEqual(expected_text, result_text)
        self.assertEqual(expected_highlights, result_highlights)

    def test_should_cut_text_w_highlights_in_title_multiple(self):
        text = 'this text contains highlights\r\nand new lines so that i can test the utility method\r\n' \
               'some additional lines are added for longer text'
        highlights = [{'start': 10, 'end': 14}]

        expected_text = text[:29]
        expected_highlights = highlights
        result_text, result_highlights = cut_text_with_highlights_if_present(text, highlights)
        self.assertEqual(expected_text, result_text)
        self.assertEqual(expected_highlights, result_highlights)

    def test_should_cut_text_w_highlights_in_body(self):
        text = 'this text contains highlights\r\nand new lines so that i can test the utility method\r\n' \
               'some additional lines are added for longer text'
        highlights = [{'start': 45, 'end': 47}, {'start': 100, 'end': 105}]

        expected_text = text[31:]
        expected_highlights = highlights
        result_text, result_highlights = cut_text_with_highlights_if_present(text, highlights)
        self.assertEqual(expected_text, result_text)
        self.assertEqual(expected_highlights, result_highlights)

    def test_should_cut_text_w_highlights_in_body_html_mixed_entities(self):
        text = 'this text contains highlights&#10;and new lines so that i can test the utility method\n' \
               'some additional lines are added for longer text'
        highlights = [{'start': 45, 'end': 47}, {'start': 100, 'end': 105}]

        expected_text = text[34:]
        expected_highlights = highlights
        result_text, result_highlights = cut_text_with_highlights_if_present(text, highlights)
        self.assertEqual(expected_text, result_text)
        self.assertEqual(expected_highlights, result_highlights)

    def test_should_cut_text_w_highlights_in_body_html_mixed_entities_swapped(self):
        text = 'this text contains highlights\nand new lines so that i can test the utility method&#10;' \
               'some additional lines are added for longer text'
        highlights = [{'start': 45, 'end': 47}, {'start': 100, 'end': 105}]

        expected_text = text[30:]
        expected_highlights = highlights
        result_text, result_highlights = cut_text_with_highlights_if_present(text, highlights)
        self.assertEqual(expected_text, result_text)
        self.assertEqual(expected_highlights, result_highlights)

    def test_should_cut_text_w_highlights_in_title_highlight_overlaps_cut(self):
        text = 'this text contains highlights\r\nand new lines so that i can test the utility method\r\n' \
               'some additional lines are added for longer text'
        highlights = [{'start': 20, 'end': 31}]

        expected_text = text[:29]
        expected_highlights = [{'start': 20, 'end': 29}]
        result_text, result_highlights = cut_text_with_highlights_if_present(text, highlights)
        self.assertEqual(expected_text, result_text)
        self.assertEqual(expected_highlights, result_highlights)

    def test_should_cut_text_w_highlights_in_title_highlight_borders_cut(self):
        text = 'this text contains highlights\r\nand new lines so that i can test the utility method\r\n' \
               'some additional lines are added for longer text'
        highlights = [{'start': 20, 'end': 28}]

        expected_text = text[:29]
        expected_highlights = [{'start': 20, 'end': 28}]
        result_text, result_highlights = cut_text_with_highlights_if_present(text, highlights)
        self.assertEqual(expected_text, result_text)
        self.assertEqual(expected_highlights, result_highlights)

    def test_should_cut_text_w_multiple_highlights_in_title_highlight_overlaps_cut(self):
        text = 'this text contains highlights\r\nand new lines so that i can test the utility method\r\n' \
               'some additional lines are added for longer text'
        highlights = [{'start': 5, 'end': 9}, {'start': 20, 'end': 31}]

        expected_text = text[:29]
        expected_highlights = [{'start': 5, 'end': 9}, {'start': 20, 'end': 29}]
        result_text, result_highlights = cut_text_with_highlights_if_present(text, highlights)
        self.assertEqual(expected_text, result_text)
        self.assertEqual(expected_highlights, result_highlights)

    def test_should_cut_text_w_highlights_in_body_overlaps_cut(self):
        text = 'this text contains highlights\r\nand new lines so that i can test the utility method\r\n' \
               'some additional lines are added for longer text'
        highlights = [{'start': 30, 'end': 36}]

        expected_text = text[31:]
        expected_highlights = [{'start': 31, 'end': 36}]
        result_text, result_highlights = cut_text_with_highlights_if_present(text, highlights)
        self.assertEqual(expected_text, result_text)
        self.assertEqual(expected_highlights, result_highlights)

    def test_should_cut_text_w_multiple_highlights_in_body_overlaps_cut(self):
        text = 'this text contains highlights\r\nand new lines so that i can test the utility method\r\n' \
               'some additional lines are added for longer text'
        highlights = [{'start': 30, 'end': 36}, {'start': 100, 'end': 105}]

        expected_text = text[31:]
        expected_highlights = [{'start': 31, 'end': 36}, {'start': 100, 'end': 105}]
        result_text, result_highlights = cut_text_with_highlights_if_present(text, highlights)
        self.assertEqual(expected_text, result_text)
        self.assertEqual(expected_highlights, result_highlights)

    def test_should_cut_text_w_multiple_highlights_in_body_borders_cut(self):
        text = 'this text contains highlights\r\nand new lines so that i can test the utility method\r\n' \
               'some additional lines are added for longer text'
        highlights = [{'start': 31, 'end': 36}, {'start': 100, 'end': 105}]

        expected_text = text[31:]
        expected_highlights = [{'start': 31, 'end': 36}, {'start': 100, 'end': 105}]
        result_text, result_highlights = cut_text_with_highlights_if_present(text, highlights)
        self.assertEqual(expected_text, result_text)
        self.assertEqual(expected_highlights, result_highlights)

    def test_should_cut_text_w_highlights_in_body_html_single_entities(self):
        text = 'this text contains highlights&#10;and new lines so that i can test the utility method&#10;' \
               'some additional lines are added for longer text'
        highlights = [{'start': 45, 'end': 47}, {'start': 100, 'end': 105}]

        expected_text = text[34:]
        expected_highlights = highlights
        result_text, result_highlights = cut_text_with_highlights_if_present(text, highlights)
        self.assertEqual(expected_text, result_text)
        self.assertEqual(expected_highlights, result_highlights)

    def test_should_cut_text_w_highlights_in_body_html_double_entities(self):
        text = 'this text contains highlights&#13;&#10;and new lines so that i can test the utility method&#13;&#10;' \
               'some additional lines are added for longer text'
        highlights = [{'start': 45, 'end': 47}, {'start': 100, 'end': 105}]

        expected_text = text[39:]
        expected_highlights = highlights
        result_text, result_highlights = cut_text_with_highlights_if_present(text, highlights)
        self.assertEqual(expected_text, result_text)
        self.assertEqual(expected_highlights, result_highlights)

    def test_should_cut_text_w_highlights_in_body_html_double_entities_swapped(self):
        text = 'this text contains highlights&#10;&#13;and new lines so that i can test the utility method&#10;&#13;' \
               'some additional lines are added for longer text'
        highlights = [{'start': 45, 'end': 47}, {'start': 100, 'end': 105}]

        expected_text = text[39:]
        expected_highlights = highlights
        result_text, result_highlights = cut_text_with_highlights_if_present(text, highlights)
        self.assertEqual(expected_text, result_text)
        self.assertEqual(expected_highlights, result_highlights)

    def test_should_cut_text_w_highlights_body_real_case(self):
        text = "Подетальная обклейка автомобиля пленкой такси.\r\nНаш сайт http://www.rbtax.ru/ Телефон 8(800) " \
               "707 10-75  или 8(499) 499 10-75 Группа VK: https://vk.com/rbtaxru Работа в #Яндекс такси  Отзыв " \
               "нового водителя Оклеивание отдельной детали ремонт пленки для такси. Такси ГОСТ Желтый цвет под " \
               "такси Как оклеить такси самому. Оклейка авто по ГОСТу 4000р при оклейки авто полностью скидка! " \
               "Специфика ухода за пленкой под такси. Пленка оклейка Оклейка такси по гост магниты яндекс бренд " \
               "Работа в такси без лицензии Обклейка автомобиля / Реклама на автомобилях"
        highlights = [{"end": 174, "start": 169}, {"end": 180, "start": 176}]
        expected_text = text[48:]
        expected_highlights = highlights
        result_text, result_highlights = cut_text_with_highlights_if_present(text, highlights)
        self.assertEqual(expected_text, result_text)
        self.assertEqual(expected_highlights, result_highlights)


from setuptools import setup, find_packages

setup(
    name='pyNlple',
    packages=find_packages(exclude=['scripts']),
    include_package_data=True,
    version='0.6.1',
    description='NLP procedures in python brought to you by YouScan.',
    author='Paul Khudan',
    author_email='pk@youscan.io',
    company='YouScan Limited',
    url='https://github.com/YouScan/pyNlple',
    install_requires=['ipython',
                      'numpy',
                      'scikit-learn',
                      'requests>=2.9.1',
                      'pandas>=0.19.0',
                      'gensim>=2.1.0',
                      'elasticsearch>=2.0.0,<3.0.0',
                      'pymorphy2',
                      'emoji'],

    classifiers=[
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='pynlple.tests',

)


# -*- coding: utf-8 -*-

import os.path


def abs_path(prefix_path=__file__, object_path=''):
    """
    Get the absolute path to the `object_path` using the `prefix_path`.

    Pass the __file__ variable to form absolute paths senseless to execution environment.
    If no source_path is set, then build absolute to the project folder (module.py).

    :param str prefix_path: path to be used as the full prefix. Default: path to module.py
    :param str object_path: path to the object in your project. Default: ''
    :return: absolute path to the object using `prefix_path`
    :rtype: str
    """
    return os.path.abspath(os.path.join(os.path.dirname(prefix_path), object_path))


def append_paths(*parts):
    """
    Append `parts` of the paths to form correct path. Wraps os.path.join method.

    :param str parts: elements of the future path
    :return: correctly-formed path
    :rtype: str
    """
    return os.path.join(*parts)


def file_name(file_path):
    """
    Get the name of the file from the `file_path` preserving extension
    (assumed as the last component of the path).

    :param str file_path: string path to the file
    :return: file name without extension
    :rtype: str
    """
    components = os.path.split(file_path)
    return components[-1]


def strip_extension(file_name_):
    """
    Strips the `file_name_` by cutting of the assumed extension of the file (the last suffix separated by '.').

    :param str file_name_: the name of the file with extension
    :return: the stripped name of the file without extension
    :rtype: str
    """
    pos = file_name_.rfind('.')
    if pos > 0:
        return file_name_[:pos]
    else:
        return file_name_


def stripped_file_name(file_path):
    """
    Get the stripped name of the file from the `file_path` (assumed as the last component of the path).
    Stripping includes cutting of the assumed extension of the file (the last suffix separated by dot)

    :param str file_path: the path to the file
    :return: file name without extension (last dot-separated suffix)
    :rtype: str
    """
    file_name_ = file_name(file_path)
    return strip_extension(file_name_)


def list_dir(path_):
    return os.listdir(path_)


def is_file(path_):
    return os.path.isfile(path_)


def is_folder(path_):
    return os.path.isdir(path_)


def exists(path_):
    return os.path.exists(path_)



# -*- coding: utf-8 -*-
import unittest

from pynlple.utils import DictionaryHelper


class DictionaryHelperTest(unittest.TestCase):
    """"""""

    def setUp(self):
        self.data = {'key': 'value',
                     'none_key': None,
                     'list_key': ['list_value1', 'list_value2'],
                     'dict_key': {'dict_dict_key': 'dict_dict_value', 'dict_dict_key2': 'dict_dict_value'},
                     'dict_key2': {'dict_list_key': ['dict_list_value', 'dict_list_value2'], 'dict_list_key2': ['dict_list_value', 'dict_list_value2']},
                     'dict_key3': {'dict_dict_key': {'dict_dict_dict_key': 'dict_dict_dict_value', 'dict_dict_dict_key2': 'dict_dict_dict_value'}, 'dict_dict_key2': {'dict_dict_dict_key': 'dict_dict_dict_value'}}
                     }
        self.data2 = {
            10: 15,
            20: 30,
            -10: -40
        }

    def test_should_match_key(self):
        condition = {'key'}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_empty_key(self):
        condition = {}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_none_key_None_value(self):
        condition = {'none_key': None}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_none_key_empty_value(self):
        condition = {'none_key'}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_list_key(self):
        condition = ['key', 'list_key']
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_dict_key(self):
        condition = {'dict_key'}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_not_match_None_key(self):
        condition = None
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_not_match_key_None_value(self):
        condition = {'key': None}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_match_key_value(self):
        condition = {'key': 'value'}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_not_match_wrong_key(self):
        condition = {'not_key'}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_not_match_key_extra_value(self):
        condition = {'key': ['value', 'value2']}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_not_match_wrong_key_None_value(self):
        condition = {'not_key': None}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_not_match_key_wrong_value(self):
        condition = {'key': 'not_value'}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_not_match_wrong_key_value(self):
        condition = {'not_key': 'value'}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_match_list_key_first_value(self):
        condition = {'list_key': 'list_value1'}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_list_key_all_values(self):
        condition = {'list_key': ['list_value1', 'list_value2']}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_not_match_list_key_right_and_wrong_values(self):
        condition = {'list_key': ['list_value1', 'not_list_value2']}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_not_match_list_key_wrong_and_right_values(self):
        condition = {'list_key': ['not_list_value1', 'list_value2']}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_match_dict_key_dict_dict_key(self):
        condition = {'dict_key': 'dict_dict_key'}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_not_match_dict_key_wrong_dict_dict_key(self):
        condition = {'dict_key': 'not_dict_dict_key'}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_match_dict_key_dict_dict_all_keys(self):
        condition = {'dict_key': ['dict_dict_key', 'dict_dict_key2']}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_dict_key_dict_dict_all_tuple_keys(self):
        condition = {'dict_key': ('dict_dict_key', 'dict_dict_key2')}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_dict_key_dict_dict_all_tuple_keys_and_value(self):
        condition = {'dict_key': ('dict_dict_key', {'dict_dict_key2': 'dict_dict_value'})}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_not_match_dict_key_dict_dict_tuple_wrong_keys(self):
        condition = {'dict_key': ('dict_dict_key', 'not_dict_dict_key2')}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_match_dict_key_dict_dict_all_tuple_keys_and_wrong_value(self):
        condition = {'dict_key': ('dict_dict_key', {'dict_dict_key2': 'not_dict_dict_value'})}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_not_match_dict_key_dict_dict_partially_wrong_keys(self):
        condition = {'dict_key': ['not_dict_dict_key', 'dict_dict_key2']}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_not_match_dict_key_dict_dict_partially_wrong_keys_2(self):
        condition = {'dict_key': ['dict_dict_key', 'not_dict_dict_key2']}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_match_dict_key_dict_list_key(self):
        condition = {'dict_key2': 'dict_list_key'}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_dict_key_dict_list_keys(self):
        condition = {'dict_key2': ['dict_list_key', 'dict_list_key2']}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_dict_key_dict_list_key_dict_list_value(self):
        condition = {'dict_key2': {'dict_list_key': 'dict_list_value'}}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_dict_key_dict_list_key_dict_list_values(self):
        condition = {'dict_key2': {'dict_list_key': ['dict_list_value', 'dict_list_value2']}}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_not_match_dict_key_dict_list_key_dict_list_partially_wrong_values(self):
        condition = {'dict_key2': {'dict_list_key': ['dict_list_value', 'not_dict_list_value2']}}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_not_match_dict_key_dict_list_key_dict_list_partially_wrong_values2(self):
        condition = {'dict_key2': {'dict_list_key': ['not_dict_list_value', 'dict_list_value2']}}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_not_match_dict_key_dict_list_partially_wrong_keys(self):
        condition = {'dict_key2': ['dict_list_key', 'not_dict_list_key2']}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_match_identical_sub_keys_and_values(self):
        condition = {'dict_key2': {'dict_list_key': ['dict_list_value', 'dict_list_value2'], 'dict_list_key2': ['dict_list_value', 'dict_list_value2']}}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_not_match_almost_identical_keys_and_values(self):
        condition = {'dict_key2': {'dict_list_key': ['dict_list_value', 'not_dict_list_value2'], 'dict_list_key2': ['dict_list_value', 'dict_list_value2']}}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_not_match_almost_identical_keys_and_values2(self):
        condition = {'dict_key2': ['dict_list_key', {'dict_list_key2': ['dict_list_value', 'not_dict_list_value2']}]}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_not_match_almost_identical_keys_and_values3(self):
        condition = {'dict_key2': {'not_dict_list_key': ['dict_list_value', 'dict_list_value2'], 'dict_list_key2': ['dict_list_value', 'dict_list_value2']}}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_match_dict_dict_dict_key(self):
        condition = {'dict_key3': {'dict_dict_key': 'dict_dict_dict_key'}}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_dict_dict_dict_keys(self):
        condition = {'dict_key3': {'dict_dict_key': ['dict_dict_dict_key', 'dict_dict_dict_key2']}}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_dict_dict_dict_keys_and_one_value(self):
        condition = {'dict_key3': {'dict_dict_key': ['dict_dict_dict_key', {'dict_dict_dict_key2': 'dict_dict_dict_value'}]}}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_dict_dict_dict_keys_and_one_value_and_dict_dict_key_and_dict_dict_dict_key(self):
        condition = {'dict_key3': {'dict_dict_key': ['dict_dict_dict_key', {'dict_dict_dict_key2': 'dict_dict_dict_value'}], 'dict_dict_key2': 'dict_dict_dict_key'}}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_identical_full_keys_and_values(self):
        condition = {'key': 'value',
                     'list_key': ['list_value1', 'list_value2'],
                     'dict_key': {'dict_dict_key': 'dict_dict_value', 'dict_dict_key2': 'dict_dict_value'},
                     'dict_key2': {'dict_list_key': ['dict_list_value', 'dict_list_value2'], 'dict_list_key2': ['dict_list_value', 'dict_list_value2']},
                     'dict_key3': {'dict_dict_key': {'dict_dict_dict_key': 'dict_dict_dict_value', 'dict_dict_dict_key2': 'dict_dict_dict_value'}, 'dict_dict_key2': {'dict_dict_dict_key': 'dict_dict_dict_value'}}
                     }
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_not_match_identical_full_keys_and_values_but_with_extra_field(self):
        condition = {'extra_key': None,
                     'key': 'value',
                     'list_key': ['list_value1', 'list_value2'],
                     'dict_key': {'dict_dict_key': 'dict_dict_value', 'dict_dict_key2': 'dict_dict_value'},
                     'dict_key2': {'dict_list_key': ['dict_list_value', 'dict_list_value2'], 'dict_list_key2': ['dict_list_value', 'dict_list_value2']},
                     'dict_key3': {'dict_dict_key': {'dict_dict_dict_key': 'dict_dict_dict_value', 'dict_dict_dict_key2': 'dict_dict_dict_value'}, 'dict_dict_key2': {'dict_dict_dict_key': 'dict_dict_dict_value'}}
                     }
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_match_integer_key(self):
        condition = {10}
        matches = DictionaryHelper.matches(condition, self.data2)
        self.assertTrue(matches)

    def test_should_match_integer_negative_key(self):
        condition = {-10}
        matches = DictionaryHelper.matches(condition, self.data2)
        self.assertTrue(matches)

    def test_should_not_match_integer_wrong_key(self):
        condition = {15}
        matches = DictionaryHelper.matches(condition, self.data2)
        self.assertFalse(matches)

    def test_should_match_integer_key_and_value(self):
        condition = {10: 15}
        matches = DictionaryHelper.matches(condition, self.data2)
        self.assertTrue(matches)

    def test_should_match_integer_keys_and_values(self):
        condition = {10: 15, 20: 30}
        matches = DictionaryHelper.matches(condition, self.data2)
        self.assertTrue(matches)

    def test_should_not_match_integer_key_and_wrong_value(self):
        condition = {10: 16}
        matches = DictionaryHelper.matches(condition, self.data2)
        self.assertFalse(matches)

    def test_should_not_match_integer_keys_and_partially_wrong_values(self):
        condition = {10: 15, 20: 40}
        matches = DictionaryHelper.matches(condition, self.data2)
        self.assertFalse(matches)

    def test_should_match_with_function_condition(self):
        condition = {10: lambda x: x < 20}
        matches = DictionaryHelper.matches(condition, self.data2)
        self.assertTrue(matches)

    def test_should_not_match_with_function_condition(self):
        condition = {10: lambda x: x < 10}
        matches = DictionaryHelper.matches(condition, self.data2)
        self.assertFalse(matches)

    def test_should_match_with_function_string_equals_condition(self):
        condition = {'key': lambda x: x == 'value'}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_not_match_with_function_string_not_equals_condition(self):
        condition = {'key': lambda x: x == 'value2'}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_match_with_function_string_contains_condition(self):
        condition = {'key': lambda x: 'v' in x and 'x' not in x}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_with_function_long_key_path(self):
        condition = {'dict_key3': {'dict_dict_key': lambda x: 'dict_dict_dict_key' in x}}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_with_function_long_key_path_simple_condition(self):
        condition = {'dict_key3': {'dict_dict_key': lambda x: len(x) > 1}}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_with_function_simple_condition_coupled_default_condition(self):
        condition = {'key': (lambda x: 'v' in x and 'x' not in x, 'value')}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_match_with_function_simple_condition_added_default_condition(self):
        condition = {'key': lambda x: 'v' in x}, {'key': 'value'}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_not_match_with_function_simple_condition_added_default_fault_condition(self):
        condition = {'key': lambda x: 'v' in x}, {'key': 'value2'}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

    def test_should_match_with_function_complex_condition(self):
        condition = {'list_key': lambda x: x == ['list_value1', 'list_value2']}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertTrue(matches)

    def test_should_not_match_with_function_complex_condition(self):
        condition = {'list_key': lambda x: x == ['list_value3', 'list_value2']}
        matches = DictionaryHelper.matches(condition, self.data)
        self.assertFalse(matches)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
from abc import ABC, abstractmethod
from itertools import chain
from pynlple.exceptions import DataSourceException


class Source(ABC):

    def __repr__(self, *args, **kwargs):
        return '{}({})'.format(str(self.__class__.__name__), repr(vars(self)))


class SkippingSource(Source):

    def __init__(self):
        self.__skips = None

    @property
    def skips(self):
        return self.__skips

    def skip(self, skip):
        if skip is None or skip < 0:
            raise DataSourceException('SkippingSource cannot skip None or negative value.')
        self.__skips = skip
        self._skip(skip)
        return self

    @abstractmethod
    def _skip(self, amount):
        raise NotImplementedError('Private method not implemented.')


class TakingSource(Source):

    def __init__(self):
        self.__takes = None

    @property
    def takes(self):
        return self.__takes

    def take(self, take):
        if take is None or take < 0:
            raise DataSourceException('TakingSource cannot take None or negative value.')
        self.__takes = take
        self._take(take)
        return self

    @abstractmethod
    def _take(self, amount):
        raise NotImplementedError('Private method not implemented.')


class SequentialSource(Source):

    def __init__(self):
        self.__slides = None
        self.__position = None
        self.__slided = 0

    @property
    def slides(self):
        return self.__slides

    @property
    def position(self):
        return self.__position

    @property
    def slided(self):
        return self.__slided

    def slide(self, slide):
        if slide is None or slide < 0:
            raise DataSourceException('SequentialSource cannot take None or negative value.')
        self.__slides = slide
        self._slide(slide)
        return self

    def reset(self):
        self.__slides = None
        self.__position = None
        self.__slided = 0
        return self

    @abstractmethod
    def _slide(self, slide):
        raise NotImplementedError('Private method not implemented.')

    def _move(self, to_position, count):
        if to_position is None:
            raise ValueError('Cannot move to None position.')
        if count is None or count < 0:
            raise ValueError('Cannot add None or negative count.')
        self.__position = to_position
        self.__slided = self.slided + count
        self._slide(self.__slides)
        return self


class BulkSource(TakingSource):

    logger = logging.getLogger(__name__)

    def _take(self, amount):
        pass

    def __init__(self, sequential_source, bulk_size=1000, reset_source=True):
        self.sequential_source = sequential_source
        if not bulk_size:
            raise ValueError('Bulk size cannot be None.')
        if bulk_size <= 0:
            raise ValueError('Bulk size must be positive number; it is: %d', bulk_size)
        self.bulk = bulk_size
        self.reset_source = reset_source
        super().__init__()

    def get_data(self):
        source = self.sequential_source
        if self.reset_source:
            source = source.reset()
            BulkSource.logger.info('Start draining sequentially %s from source %s.',
                                   str(self.takes) if self.takes is not None else 'all data',
                                   repr(source))
        else:
            BulkSource.logger.info('Continue draining sequentially %s from %s, position %s.',
                                   str(self.takes) if self.takes is not None else 'all data',
                                   repr(source), str(source.position))
        all_data = []
        try:
            if self.takes is None or self.takes > 0:
                pull = True
            else:
                pull = False
            while pull:
                take = self.__get_window_size(source.slided)
                BulkSource.logger.info('Getting %d-%d bulk from sequential source',
                                       source.slided, (source.slided + take))

                window_data = source.slide(take).get_data()
                all_data.append(window_data)

                got = len(window_data)
                if self.takes is not None and self.takes > 0:
                    pull = source.slided < self.takes and got == take
                else:
                    pull = got == take
            if self.takes is not None:
                SkippingBulkSource.logger.info('Stopped draining from source at position %s, requested count: %d',
                                               str(source.position), source.slided)
            else:
                SkippingBulkSource.logger.info('Source seems to be exhausted at position: %s, count: %d',
                                               str(source.position), source.slided)
        except Exception as e:
            raise DataSourceException('Error while dumping bulk %d-%d from %s:\n%s' %
                                      (source.slided, (source.slided + take), repr(self), repr(e)))
        return list(chain.from_iterable(all_data))

    def __get_window_size(self, got_total):
        # If there are limits on amount to take, check if we reached them
        if self.takes:
            to_limit_leftover = self.takes - got_total
            if self.bulk > to_limit_leftover:
                return to_limit_leftover
            else:
                return self.bulk
        else:
            return self.bulk


class SkippingBulkSource(TakingSource):

    logger = logging.getLogger(__name__)

    def _take(self, amount):
        pass

    def __init__(self, skip_n_taking_source, bulk_size=5000):
        self.skip_n_taking_source = skip_n_taking_source
        self.bulk = bulk_size
        super().__init__()

    def get_data(self):
        SkippingBulkSource.logger.info('Start draining %s from %s.',
                                       str(self.takes) if self.takes is not None else 'all data',
                                       repr(self.skip_n_taking_source))
        all_data = []
        total = 0
        try:
            if self.takes is None or self.takes > 0:
                pull = True
            else:
                pull = False
            while pull:
                take = self.__get_window_size(total)
                SkippingBulkSource.logger.info('Getting %d-%d bulk from the skipping source'
                                               % (total, total+take))
                data = self.skip_n_taking_source.skip(total).take(take).get_data()
                all_data.append(data)

                got = len(data)
                total += got
                pull = total < self.takes if self.takes is not None else got == take

            if self.takes is not None:
                SkippingBulkSource.logger.info('Stopped draining from source at requested point: %d',
                                               total)
            else:
                SkippingBulkSource.logger.info('Source seems to be exhausted at count: %d',
                                               total)
        except Exception as e:
            raise DataSourceException('Error while dumping bulk %d-%d from %s:\n%s' %
                                      (total, (total + take), repr(self), repr(e)))
        return list(chain.from_iterable(all_data))

    def __get_window_size(self, got_total):
        # If there are limits on amount to take, check if we reached them
        if self.takes:
            to_limit_leftover = self.takes - got_total
            if self.bulk > to_limit_leftover:
                return to_limit_leftover
            else:
                return self.bulk
        else:
            return self.bulk



# -*- coding: utf-8 -*-
import unittest

from pynlple.processing.dictionary import DictionaryLookUp
from pynlple.processing.text import ClassTokenFilter, RuleTokenizer, DictionaryBasedTagger


class TokenizerTest(unittest.TestCase):

    def test_should_tokenize_words(self):
        input_string = '!These words, un-like-ly said here(sic!), should be tokenized. ' \
                  'Ukrainska mova takoj z\'yavliaetsia tyt.'
        tokenizer = RuleTokenizer()
        expected = '! These words , un-like-ly said here ( sic ! ) , should be tokenized . ' \
                   'Ukrainska mova takoj z\'yavliaetsia tyt .'.split()

        result = tokenizer.tokenize(input_string)
        self.assertEqual(expected, result)

    def test_should_tokenize_words_w_cyrillics(self):
        input_string = 'за 2011 год). Вот полный список торговых марок от Nestle, которые можно найти в любом российском магазине: Nescafé, КитКат,'
        tokenizer = RuleTokenizer()
        expected = 'за 2011 год ) . Вот полный список торговых марок от Nestle , которые можно найти в любом российском магазине : Nescafé , КитКат ,'.split()

        result = tokenizer.tokenize(input_string)
        self.assertEqual(expected, result)


class ClassTokenFilterTest(unittest.TestCase):

    def setUp(self):
        self.mappings = [
            ('a b', 'c1'),
            ('a b c', 'c1'),
            ('.', 's'),
            (',', 's'),
            ('d', 'c2'),
            ('e', 'c2'),
            ('f', 'c3'),
            ]
        self.dictionary_lookup = DictionaryLookUp(self.mappings)
        self.tagger = DictionaryBasedTagger(self.dictionary_lookup)

    def test_should_not_filter_no_representatives(self):
        filter_ = ClassTokenFilter(self.tagger, ['s'])
        tokens = 'y n n y n'.split()

        expected_tokens = 'y n n y n'.split()
        self.assertEqual(expected_tokens, filter_.filter(tokens))

    def test_should_filter_s_class_no_other_classes(self):
        filter_ = ClassTokenFilter(self.tagger, ['s'])
        tokens = 'y n . n y , n'.split()

        expected_tokens = 'y n n y n'.split()
        self.assertEqual(expected_tokens, filter_.filter(tokens))

    def test_should_filter_s_class_with_other_classes(self):
        filter_ = ClassTokenFilter(self.tagger, ['s'])
        tokens = 'y n a b . n d e f y , n a b c'.split()

        expected_tokens = 'y n a b n d e f y n a b c'.split()
        self.assertEqual(expected_tokens, filter_.filter(tokens))

    def test_should_filter_two_classes_no_other_classes(self):
        filter_ = ClassTokenFilter(self.tagger, ['s', 'c3'])
        tokens = 'y f n . n f y , n f'.split()

        expected_tokens = 'y n n y n'.split()
        self.assertEqual(expected_tokens, filter_.filter(tokens))

    def test_should_filter_two_classes_with_other_classes(self):
        filter_ = ClassTokenFilter(self.tagger, ['s', 'c3'])
        tokens = 'y n f a b . n d e f y , n f a b c'.split()

        expected_tokens = 'y n a b n d e y n a b c'.split()
        self.assertEqual(expected_tokens, filter_.filter(tokens))

    def test_should_filter_all_known_classes(self):
        filter_ = ClassTokenFilter(self.tagger)
        tokens = 'y n f a b . n d e f y , n f a b c'.split()

        expected_tokens = 'y n n y n'.split()
        self.assertEqual(expected_tokens, filter_.filter(tokens))
